#pragma strict

var movement_accel = 1.0;
var movement_speed = 0.0;
var max_speed = 4.0;
var position : Vector3;
var rotation : Quaternion;
var display_rotation : Quaternion;
private var tilt_amount = 0.0;
private var tilt_vel = 0.0;
var turn_accel = 50.0;
private var turn_speed = 0.0;
var steam_sound : AudioClip;
var explode_sound : AudioClip;
var steam_sound_delay = 1.0;
private var cur_steam_sound_delay = steam_sound_delay;
var velocity = Vector3(0.0, 0.0, 0.0);
var drive_velocity = Vector3(0.0, 0.0, 0.0);
private var shadow_object : GameObject;
var shadow_prefab : GameObject;
var next_car : GameObject;
var prev_car : GameObject;
var car_length = 0.9;
private var angular_velocity = 0.0;
var is_engine = false;
var gun_offset = 0.0;
var player_controlled = false;
var target : GameObject;
var damage = 0.0;
var heat = 0.0;
private var being_heated = false;
var damage_effect : GameObject;
private var damage_effect_object : GameObject;
var splode_effect : GameObject;
var destroyed = false;
var attach_fix = 0.0;
var max_attach_fix = 0.05;

function Start () {
	position = transform.position;
	rotation = transform.rotation;
	display_rotation = Quaternion.identity;
	if(shadow_prefab){
		shadow_object = Instantiate(shadow_prefab, position, Quaternion.identity);
		shadow_object.GetComponent(Renderer).material.color.a = 0.5;
	}
	if(damage_effect){
		damage_effect_object = Instantiate(damage_effect, position, Quaternion.identity);
	}
	var gun_battery_script = gameObject.GetComponent(GunBatteryScript);
	gun_battery_script.AddGun(Vector3(0.01+gun_offset,0.0,-0.15));
	gun_battery_script.AddGun(Vector3(0.15+gun_offset,0.0,-0.15));
	gun_battery_script.AddGun(Vector3(-0.13+gun_offset,0.0,-0.15));
	gun_battery_script.AddGun(Vector3(0.51+gun_offset,0.0,-0.15));
	gun_battery_script.AddGun(Vector3(0.65+gun_offset,0.0,-0.15));
	gun_battery_script.AddGun(Vector3(0.37+gun_offset,0.0,-0.15));
}

function Remove() {
	if(shadow_object){
		Destroy(shadow_object);
	}
	if(damage_effect_object){
		damage_effect_object.GetComponent(ParticleSystem).emissionRate = 0.0;
	}
	Destroy(gameObject);
}

function RotateOnZAxis(angle){
	rotation = Quaternion.AngleAxis(angle, Vector3(0.0,0.0,1.0)) * rotation;
}

function ApplyForceAtPoint(force : Vector3, where : Vector3){
	force.z = 0.0;
	velocity += force * Time.deltaTime;
	var right_couple = Quaternion.AngleAxis(90,Vector3(0,0,1)) * (where - transform.position);
	angular_velocity += Vector3.Dot(right_couple,force) * 0.9;
}

function Damage(how_much : float){
	if(player_controlled){
		GameObject.Find("Level").GetComponent(LevelScript).UpdateHUDDamaged();
	}
	damage = Mathf.Min(1.0, damage + how_much);
	if(damage == 1.0 && !destroyed){
		destroyed = true;
		if(player_controlled){
			GameObject.Find("Level").GetComponent(LevelScript).PlayerDied(1);
		} else {
			if(is_engine){
				GameObject.Find("Level").GetComponent(LevelScript).EnemyDied();
			}
		}
		AudioSource.PlayClipAtPoint(explode_sound, position, 2.0);
		var explode_effect_object = Instantiate(splode_effect, position, Quaternion.identity);
		explode_effect_object.transform.position = position;
		explode_effect_object.transform.rotation = rotation * Quaternion.AngleAxis(90.0,Vector3(0.0,0.0,1.0)) * Quaternion.AngleAxis(180.0,Vector3(-1.0,0.0,0.0));
		explode_effect_object.GetComponent(CleanUpSelf).lifetime = 3.0;
	}
}

function ApplyImpulseAtPoint(force : Vector3, where : Vector3){
	force.z = 0.0;
	position += force;
}

function Update () {
	if(!destroyed && (is_engine || next_car)){
		var gun_battery_script = gameObject.GetComponent(GunBatteryScript);
		if(player_controlled){
			if(Input.GetKeyDown("q")){
				tilt_vel += gun_battery_script.FireBattery(false);		
			}
			if(Input.GetKeyDown("e")){
				tilt_vel += gun_battery_script.FireBattery(true);
			}
		} else {
			if(target && !target.GetComponent(CarScript).destroyed){
				var target_pos = target.GetComponent(CarScript).position;
				var range = (target_pos - position).magnitude;
				if(range < 5.0){
					var right = Quaternion.AngleAxis(90,Vector3(0,0,1)) * transform.rotation * Vector3(1,0,0);
					var rightness = Vector3.Dot((target_pos - position).normalized, right);
					if(rightness > 0.8){
						tilt_vel += gun_battery_script.FireBattery(false);
					} else if(rightness < -0.8) {
						tilt_vel += gun_battery_script.FireBattery(true);
					}
				}
			}
		}
	}
}

function AttachBehind( car : GameObject ) {
	prev_car = car;
	var other_car_script = prev_car.GetComponent(CarScript);
	other_car_script.next_car = gameObject;
	if(player_controlled){
		other_car_script.player_controlled = true;
	}
	other_car_script.attach_fix = max_attach_fix;
}

function HandleCarCollision () {
	var cars : GameObject[] = GameObject.Find("Level").GetComponent(LevelScript).GetCars().ToBuiltin(GameObject);
	var near_phase_cars = new Array();
	var my_sphere_size = GetSphereSize();
	for( var car : GameObject in cars ){
		if(car == gameObject){
			continue;
		}
		var other_car_script = car.GetComponent(CarScript);
		var other_sphere_size = other_car_script.GetSphereSize();
		var dist = (other_car_script.position-position).magnitude;
		if(dist < my_sphere_size + other_sphere_size){
			near_phase_cars.Push(car);
		}
	}
	if(near_phase_cars.length == 0){
		return;
	}
	var my_spheres : Vector3[] = GetCollisionSpheres().ToBuiltin(Vector3);
	for( var car : GameObject in near_phase_cars ){
		//Debug.DrawLine(position, car.GetComponent(CarScript).position, Color.green);
		var other_spheres : Vector3[] = car.GetComponent(CarScript).GetCollisionSpheres().ToBuiltin(Vector3);
		for(var ms in my_spheres){
			for(var os in other_spheres){
				dist = (os-ms).magnitude;
				if(dist < 0.7){
					//Debug.DrawLine(ms, os, Color.red);
					var depth = 0.7 - dist;
					//ApplyForceAtPoint((os+ms)*0.5, (os-ms).normalized * -depth);
					var point = (os+ms)*0.5;
					var force = (os-ms).normalized * depth * 50.0;
					other_car_script = car.GetComponent(CarScript);
					if(car == prev_car){
						force *= 1.0 - other_car_script.attach_fix/max_attach_fix;
					}
					other_car_script.ApplyForceAtPoint(force,point);	
					ApplyForceAtPoint(-force,point);
					//Debug.DrawLine(point, point+force, Color.red);
					if(!prev_car && (next_car || is_engine) && !other_car_script.is_engine){
						var prev_couple = other_car_script.position + car.transform.rotation * Vector3(other_car_script.car_length,0,0);
						var couple = position - transform.rotation * Vector3(car_length,0.0,0.0);
						if((couple-prev_couple).magnitude < 0.6){
							AttachBehind(car);
						}
					}
				}
			}
		}
	}
}

function FixedUpdate () {
	var speed = 1.0;
	if(is_engine && !destroyed){
		if(player_controlled){
			if(Input.GetKey("w")){
				movement_speed += movement_accel * Time.deltaTime;
			}
			if(Input.GetKey("s")){
				movement_speed -= movement_accel * Time.deltaTime;
			}
		} else {
			if(target && !target.GetComponent(CarScript).destroyed){
				var target_pos = target.GetComponent(CarScript).position;
				var front = transform.rotation * Vector3(1,0,0);
				var frontness = Vector3.Dot((target_pos - position).normalized, front);
				frontness = Mathf.Clamp(frontness, -1.0, 1.0);
				movement_speed += movement_accel * Time.deltaTime * frontness;
			} else {
				var accel = movement_accel * Time.deltaTime;
				if(movement_speed > accel){
					movement_speed -= accel;
				} else if(movement_speed < -accel){
					movement_speed += accel;
				} else {
					movement_speed = 0.0;
				}
			}
		}
	} else {
		movement_speed *= 0.95;
	}
	drive_velocity = rotation * Vector3(1.0, 0.0, 0.0) * movement_speed;
	position += velocity * Time.deltaTime + drive_velocity * Time.deltaTime;
	if(is_engine && !destroyed){
		if(player_controlled){
			if(Input.GetKey("a")){
				turn_speed += turn_accel * Time.deltaTime;
			}
			if(Input.GetKey("d")){
				turn_speed -= turn_accel * Time.deltaTime;
			}
		} else {
			if(target && !target.GetComponent(CarScript).destroyed){
				target_pos = target.GetComponent(CarScript).position;
				var dir = (target_pos - position).normalized;
				if(movement_speed < 0.0){
					dir *= -1.0;
				}
				front = transform.rotation * Vector3(1,0,0);
				Debug.DrawLine(position, position+front*5, Color.red);
				Debug.DrawLine(position, position+dir*5, Color.green);
				var angle = Mathf.Atan2(front.y, front.x) * 180.0 / Mathf.PI;
				var target_rotation = Mathf.Atan2(dir.y, dir.x) * 180.0 / Mathf.PI;
				while(target_rotation > angle + 180.0){ 
					target_rotation -= 360.0;
				}
				while(target_rotation < angle - 180.0){ 
					target_rotation += 360.0;
				}
				var range = (target_pos - position).magnitude;
				if(range < 5.0){
					if(angle > target_rotation){
						target_rotation += 90.0;
					} else if(angle < target_rotation){
						target_rotation -= 90.0;
					}
				}
				if(angle > target_rotation){
					turn_speed -= turn_accel * Time.deltaTime;
				} else if(angle < target_rotation){
					turn_speed += turn_accel * Time.deltaTime;
				}
			}
		}
	}
	turn_speed *= 0.95;
	RotateOnZAxis(Time.deltaTime * turn_speed * movement_speed);
	tilt_vel -= turn_speed * movement_speed * 0.03;
	movement_speed = Mathf.Max(-max_speed, Mathf.Min(max_speed, movement_speed));
	
	tilt_amount += tilt_vel * Time.deltaTime;
	tilt_vel += tilt_amount * -10.0 * Time.deltaTime;
	tilt_vel *= 0.95;
	display_rotation = Quaternion.AngleAxis(tilt_amount-90,Vector3(1,0,0));
	transform.position = position;
	transform.position.z += 0.15;
	transform.rotation = rotation * display_rotation;
	if(shadow_object){
		shadow_object.transform.position = position;
		shadow_object.transform.position.z += 0.15;
		shadow_object.transform.rotation = rotation * Quaternion.AngleAxis(90.0,Vector3(-1.0,0.0,0.0));
		shadow_object.transform.position += rotation * Vector3(-0.05,-0.05,0.0);
	}
	if(damage_effect_object){
		damage_effect_object.transform.position = position;
		damage_effect_object.transform.rotation = rotation * Quaternion.AngleAxis(90.0,Vector3(0.0,0.0,1.0)) * Quaternion.AngleAxis(90.0,Vector3(-1.0,0.0,0.0));
		damage_effect_object.GetComponent(CleanUpSelf).lifetime = 3.0;
		if(!destroyed){
			damage_effect_object.GetComponent(ParticleSystem).emissionRate = Mathf.Max(heat, damage) * 100;
		} else {
			damage_effect_object.GetComponent(ParticleSystem).emissionRate = 0;
		}
	}
	
	if(is_engine && !destroyed){
		cur_steam_sound_delay -= Mathf.Min(3.0,Mathf.Abs(movement_speed)) * Time.deltaTime;
		if(cur_steam_sound_delay <= 0.0){
			cur_steam_sound_delay = steam_sound_delay;
			AudioSource.PlayClipAtPoint(steam_sound, position, 0.3 - 0.2 * Mathf.Min(1.0,movement_speed*0.3));
		}
	}
	
	if(next_car){
		var next_car_script = next_car.GetComponent(CarScript);
		var next_couple = next_car_script.position - next_car.transform.rotation * Vector3(next_car_script.car_length,0,0);
		var couple = position + transform.rotation * Vector3(car_length,0.0,0.0);
		//Debug.DrawLine(couple, next_couple, Color.red, 0, false);
		var force = (next_couple - couple) * 200.0;
		if(attach_fix > 0.0){
			position += (next_couple - couple)*attach_fix/max_attach_fix;
			attach_fix = Mathf.Max(0.0, attach_fix - Time.deltaTime);
			force *= 1.0 - attach_fix/max_attach_fix;
		}
		next_car_script.ApplyForceAtPoint(-force, next_couple);
		ApplyForceAtPoint(force, couple);
	}
	RotateOnZAxis(Time.deltaTime * angular_velocity);
	angular_velocity *= 0.95;
	velocity *= 0.92;	
	GetComponent(Rigidbody).velocity = velocity + drive_velocity;	
	
	HandleCarCollision();
	
	if(is_engine && player_controlled && !destroyed){
		GameObject.Find("Level").GetComponent(LevelScript).UpdateHUDHeat(heat);
	}
	if(!being_heated){
		heat = Mathf.Max(0.0, heat - Time.deltaTime * 0.3);
	}
	if(damage < 1.0){
		damage = Mathf.Max(0.0, damage - Time.deltaTime * 0.1);
	}
	for (var child : Transform in transform) {
		child.gameObject.GetComponent(Renderer).material.color.r = 1.0;
		child.gameObject.GetComponent(Renderer).material.color.g = (1.0 - heat);
		child.gameObject.GetComponent(Renderer).material.color.b = (1.0 - heat);
		child.gameObject.GetComponent(Renderer).material.color.r *= 1.0 - damage * 0.5;
		child.gameObject.GetComponent(Renderer).material.color.g *= 1.0 - damage * 0.5;
		child.gameObject.GetComponent(Renderer).material.color.b *= 1.0 - damage * 0.5;
	}
	being_heated = false;
}

function Heat( amount : float ) {
	being_heated = true;
	heat = Mathf.Min(1.0, heat + amount * 0.1);
	if(heat == 1.0){
		destroyed = true;
		GameObject.Find("Level").GetComponent(LevelScript).PlayerDied(0);
	}
}

function GetCollisionSpheres() : Array {
	var array = new Array();
	var front = transform.rotation * Vector3(1,0,0);
	for(var i=-2; i<=2; ++i){
		var point = transform.position + front * i * 0.3 * car_length;
		point.z = 0.0;
		array.Push(point);
		//Gizmos.DrawWireSphere (transform.position + front * i * 0.3, 0.35);
	}
	return array;
}

function GetSphereSize() {
	return car_length * 2.0;
}

function OnDrawGizmos() {
	var array : Vector3[] = GetCollisionSpheres().ToBuiltin(Vector3);
	for(var i=0; i<array.length; ++i){
		//Gizmos.DrawWireSphere (array[i], 0.35);
	}
	//Gizmos.DrawWireSphere (position, car_length * 2.0);
	Gizmos.DrawWireSphere(position - transform.rotation * Vector3(car_length,0.0,0.0), 0.3);
	Gizmos.DrawWireSphere(position + transform.rotation * Vector3(car_length,0.0,0.0), 0.3);
						
}