#pragma strict

var target : GameObject;
var splat_sound : AudioClip;
var angle = 0.0;
private var shadow_object : GameObject;
var shadow_prefab : GameObject;
var explode_particles : GameObject;
private var speed = 0.0;
var flamethrower_prefab : GameObject;
private var flamethrower_object : GameObject;

function Start () {
	angle = Random.Range(0.0, 360.0);
	if(shadow_prefab){
		shadow_object = Instantiate(shadow_prefab, transform.position, transform.rotation);
		shadow_object.GetComponent(Renderer).material.color.a = 0.5;
	}
}

function Remove() {
	if(shadow_object){
		Destroy(shadow_object);
	}
	if(flamethrower_object){
		flamethrower_object.GetComponent(ParticleSystem).emissionRate = 0;
	}
	Destroy(gameObject);
}

function Die (velocity : Vector3) {
	AudioSource.PlayClipAtPoint(splat_sound, transform.position, 1.0);
	var explosion = Instantiate(explode_particles, transform.position, Quaternion.identity);
	explosion.GetComponent(CleanUpSelf).velocity = velocity * 0.5;
	Instantiate(explode_particles, transform.position, Quaternion.identity);
	Remove();
	GameObject.Find("Level").GetComponent(LevelScript).BadGuyDied(gameObject);
}

function GetRotation() : Quaternion {
	return Quaternion.EulerAngles(0.0,0.0,angle / 180.0 * Mathf.PI);
}

function GetForwards() : Vector3 {
	return GetRotation() * Vector3(1.0,0.0,0.0);
}

function GetFlamethrowerPosition() : Vector3 {
	return transform.position + GetForwards() * 0.4 + Vector3(0.0,0.0,-0.17);
}

function GetFlamethrowerRotation() : Quaternion {
	return transform.rotation * Quaternion.AngleAxis(90.0,Vector3(0.0,1.0,0.0));
}

function FixedUpdate () {
	if(target){
		var target_pos = target.transform.position + target.GetComponent(CarScript).velocity;
		var dir = (target_pos - transform.position).normalized;
		var target_rotation = Mathf.Atan2(dir.y, dir.x) * 180.0 / Mathf.PI;
		while(target_rotation > angle + 180.0){ 
			target_rotation -= 360.0;
		}
		while(target_rotation < angle - 180.0){ 
			target_rotation += 360.0;
		}
		var rotate_amount = 60.0 * Time.deltaTime;
		var old_angle = angle;
		if(Mathf.Abs(angle - target_rotation) < rotate_amount){
			angle = target_rotation;	
		} else if(angle > target_rotation){
			angle -= rotate_amount;
		} else if(angle < target_rotation){
			angle += rotate_amount;
		}
	}
	
	var target_speed = 1.0;
	if(target){
		var dist_from_target = (target_pos - transform.position).magnitude;
		if(dist_from_target < 3.0){
			target_speed = 0.0;	
		}
		if(dist_from_target < 2.0){
			target_speed = -1.0;	
		}
	} else {
		dist_from_target = 100.0;
		target_speed = 0.0;
	}
	if(dist_from_target > 15){
		target_speed = 10.0;
	}
	speed = Mathf.Lerp(speed, target_speed, 0.1);
	animation["walk"].speed = speed * 1.3;
	
	var rotation = GetRotation();
	
	if(animation["walk"].speed > 0){
		animation["walk"].speed += Mathf.Abs(angle - old_angle);
	} else {
		animation["walk"].speed -= Mathf.Abs(angle - old_angle);
	}
	
	transform.position += GetForwards() * Time.deltaTime * speed;
	transform.rotation = rotation * Quaternion.AngleAxis(90.0,Vector3(-1.0,0.0,0.0));
	
	if(!flamethrower_object && dist_from_target < 3.0){
		flamethrower_object = Instantiate(flamethrower_prefab, GetFlamethrowerPosition(), GetFlamethrowerRotation());
	}
	if(flamethrower_object){
		flamethrower_object.transform.position = GetFlamethrowerPosition();
		flamethrower_object.transform.rotation = GetFlamethrowerRotation();
		if(dist_from_target < 3.0){
			flamethrower_object.GetComponent(CleanUpSelf).lifetime = 1.0;
			flamethrower_object.GetComponent(ParticleSystem).emissionRate = 100;
			if(Mathf.Abs(angle - target_rotation) < 30){
				target.GetComponent(CarScript).Heat(1.0 * Time.deltaTime);
			}
		} else {
			flamethrower_object.GetComponent(ParticleSystem).emissionRate = 0;
		}
	}
	
	if(shadow_object){
		shadow_object.transform.position = transform.position - Vector3(0.0,0.0,0.1);
		shadow_object.transform.rotation = transform.rotation;// * Quaternion.AngleAxis(90.0,Vector3(-1.0,0.0,0.0));
		shadow_object.transform.position += transform.rotation * Vector3(-0.05,-0.05,0.0);
	}
	
	var check_radius = 0.7;
	var hits : Collider[] = Physics.OverlapSphere (transform.position, check_radius);
	var collided : boolean = false;
	var vel;
	for (var c : Collider in hits) {
		if(c.name == "CannonBall(Clone)"){
			collided = true;
			vel = c.GetComponent(CannonBallScript).velocity;
		}
	}
	if(collided){
		Die(vel);
	}
}

function OnCollisionEnter(collision : Collision) {
    if(collision.other.name == "Player"){
		Die(collision.other.GetComponent(CarScript).velocity + 
			collision.other.GetComponent(CarScript).drive_velocity);
	}
}